<?php
namespace PerProduct\Core;

class Rule {
	/** @var integer */
	private $product_id;

	/** @var ?integer */
	private $variation_id = null;

	/** @var string */
	private $country = '*';

	/** @var string */
	private $state = '*';

	/** @var array */
	private $postcodes;

	/** @var integer */
	private $quantity;

	/** @var float */
	private $line_cost;

	/** @var float */
	private $item_cost;

	/**
	 * @return int
	 */
	public function getProductId() {
		return $this->product_id;
	}

	/**
	 * @param int $product_id
	 *
	 * @return Rule
	 */
	public function setProductId( $product_id ) {
		$this->product_id = $product_id;

		return $this;
	}

	/**
	 * @return int|null
	 */
	public function getVariationId() {
		return $this->variation_id;
	}

	/**
	 * @param int|null $variation_id
	 *
	 * @return Rule
	 */
	public function setVariationId( $variation_id ) {
		$this->variation_id = $variation_id;

		return $this;
	}

	/**
	 * @return string
	 */
	public function getCountry() {
		return $this->country;
	}

	/**
	 * @param string $country
	 *
	 * @return Rule
	 */
	public function setCountry( $country ) {
		$this->country = $country;

		return $this;
	}

	/**
	 * @return string
	 */
	public function getState() {
		return $this->state;
	}

	/**
	 * @param string $state
	 *
	 * @return Rule
	 */
	public function setState( $state ) {
		$this->state = $state;

		return $this;
	}

	/**
	 * @return array
	 */
	public function getPostcodes() {
		return $this->postcodes;
	}

	/**
	 * @param array $postcodes
	 *
	 * @return Rule
	 */
	public function setPostcodes( $postcodes ) {
		$this->postcodes = $postcodes;

		return $this;
	}

	/**
	 * @return int
	 */
	public function getQuantity() {
		return $this->quantity;
	}

	/**
	 * @param int $quantity
	 *
	 * @return Rule
	 */
	public function setQuantity( $quantity ) {
		$this->quantity = $quantity;

		return $this;
	}

	/**
	 * @return float
	 */
	public function getLineCost() {
		return $this->line_cost;
	}

	/**
	 * @param float $line_cost
	 *
	 * @return Rule
	 */
	public function setLineCost( $line_cost ) {
		$this->line_cost = $line_cost;

		return $this;
	}

	/**
	 * @return float
	 */
	public function getItemCost() {
		return $this->item_cost;
	}

	/**
	 * @param float $item_cost
	 *
	 * @return Rule
	 */
	public function setItemCost( $item_cost ) {
		$this->item_cost = $item_cost;

		return $this;
	}

	/**
	 * @param array $rule
	 *
	 * @return Rule
	 */
	public static function fromPerShippingRule($rule)
	{
		$instance = new self();

		$instance->setProductId($rule['product']['code']);

		if (isset($rule['variation'])) {
			$instance->setVariationId($rule['variation']);
		}

		$instance->setCountry($rule['country']['code']);
		$instance->setState($rule['state']);
		if (trim($rule['postcode']) !== '*') {
			$instance->setPostcodes(explode(',', $rule['postcode']));
		}
		$instance->setQuantity($rule['quantity']);
		$instance->setLineCost($rule['line_cost']);
		$instance->setItemCost($rule['item_cost']);

		return $instance;
	}

}
